@@ -75,7 +75,7 @@ class LookupMerger(UnknownMerger):
 
 
 def dict_extract_mergers(config):
-    parsed_mergers = []
+    parsed_mergers: list = []
     raw_mergers = config.pop("merge_how", None)
     if raw_mergers is None:
         raw_mergers = config.pop("merge_type", None)
@@ -127,7 +127,7 @@ def default_mergers():
 
 def construct(parsed_mergers):
     mergers_to_be = []
-    for (m_name, m_ops) in parsed_mergers:
+    for m_name, m_ops in parsed_mergers:
         if not m_name.startswith(MERGER_PREFIX):
             m_name = MERGER_PREFIX + str(m_name)
         merger_locs, looked_locs = importer.find_module(
@@ -145,8 +145,8 @@ def construct(parsed_mergers):
             mod_attr = getattr(mod, MERGER_ATTR)
             mergers_to_be.append((mod_attr, m_ops))
     # Now form them...
-    mergers = []
+    mergers: list = []
     root = LookupMerger(mergers)
-    for (attr, opts) in mergers_to_be:
+    for attr, opts in mergers_to_be:
         mergers.append(attr(root, opts))
     return root
