@@ -1,4 +1,5 @@
 """Networking-related tests."""
+
 import contextlib
 import json
 
@@ -14,10 +15,9 @@ from tests.integration_tests.releases import (
     CURRENT_RELEASE,
     IS_UBUNTU,
     JAMMY,
-    MANTIC,
     NOBLE,
 )
-from tests.integration_tests.util import verify_clean_log
+from tests.integration_tests.util import verify_clean_boot, verify_clean_log
 
 # Older Ubuntu series didn't read cloud-init.* config keys
 LXD_NETWORK_CONFIG_KEY = (
@@ -66,6 +66,13 @@ class TestNetplanGenerateBehaviorOnReboot:
         client.execute(
             "mv /var/log/cloud-init.log /var/log/cloud-init.log.bak"
         )
+        if CURRENT_RELEASE < JAMMY:
+            assert (
+                "No netplan python module. Fallback to write"
+                " /etc/netplan/50-cloud-init.yaml" in log
+            )
+        else:
+            assert "Rendered netplan config using netplan python API" in log
         netplan = yaml.safe_load(
             client.execute("cat /etc/netplan/50-cloud-init.yaml")
         )
@@ -191,7 +198,10 @@ def test_netplan_rendering(
     }
     with session_cloud.launch(launch_kwargs=launch_kwargs) as client:
         result = client.execute("cat /etc/netplan/50-cloud-init.yaml")
-        assert result.stdout.startswith(EXPECTED_NETPLAN_HEADER)
+        if CURRENT_RELEASE < JAMMY:
+            assert result.stdout.startswith(EXPECTED_NETPLAN_HEADER)
+        else:
+            assert EXPECTED_NETPLAN_HEADER not in result.stdout
         assert expected == yaml.safe_load(result.stdout)
 
 
@@ -281,19 +291,19 @@ def test_invalid_network_v2_netplan(
             "config_dict": config_dict,
         }
     ) as client:
-        # Netplan python API only available on MANTIC and later
-        if CURRENT_RELEASE < MANTIC:
+        # Netplan python API only available on JAMMY and later
+        if CURRENT_RELEASE < JAMMY:
             assert (
-                "Skipping netplan schema validation. No netplan available"
+                "Skipping netplan schema validation. No netplan API available"
             ) in client.read_from_file("/var/log/cloud-init.log")
             assert (
-                "Skipping network-config schema validation. No network schema"
-                " for version: 2"
+                "Skipping network-config schema validation for version: 2."
+                " No netplan API available."
             ) in client.execute("cloud-init schema --system")
         else:
             assert (
-                "Invalid network-config provided: Please run "
-                "'sudo cloud-init schema --system' to see the schema errors."
+                "network-config failed schema validation! You may run "
+                "'sudo cloud-init schema --system' to check the details."
             ) in client.execute("cloud-init status --format=json")
             assert (
                 "Invalid network-config /var/lib/cloud/instances/"
@@ -310,77 +320,29 @@ def test_ec2_multi_nic_reboot(setup_image, session_cloud: IntegrationCloud):
     """Tests that additional secondary NICs and secondary IPs on them are
     routable from non-local networks after a reboot event when network updates
     are configured on every boot."""
-    ec2 = session_cloud.cloud_instance.client
     with session_cloud.launch(launch_kwargs={}, user_data=USER_DATA) as client:
-        # Add secondary NIC
-        secondary_priv_ip_0 = client.instance.add_network_interface()
-        response = ec2.describe_network_interfaces(
-            Filters=[
-                {
-                    "Name": "private-ip-address",
-                    "Values": [secondary_priv_ip_0],
-                },
-            ],
+        # Add secondary NIC with two private and public ips
+        client.instance.add_network_interface(
+            ipv4_address_count=2, ipv4_public_ip_count=2
         )
-        nic_id = response["NetworkInterfaces"][0]["NetworkInterfaceId"]
-        # Add secondary IP to secondary NIC
-        association_0 = ec2.assign_private_ip_addresses(
-            NetworkInterfaceId=nic_id, SecondaryPrivateIpAddressCount=1
+
+        public_ips = client.instance.public_ips
+        assert len(public_ips) == 3, (
+            "Expected 3 public ips, one from the primary nic and 2 from the"
+            " secondary one"
         )
-        assert association_0["ResponseMetadata"]["HTTPStatusCode"] == 200
-        secondary_priv_ip_1 = association_0["AssignedPrivateIpAddresses"][0][
-            "PrivateIpAddress"
-        ]
 
-        # Assing elastic IPs
-        # Refactor after https://github.com/canonical/pycloudlib/issues/337 is
-        # completed
-        allocation_0 = ec2.allocate_address(Domain="vpc")
-        allocation_1 = ec2.allocate_address(Domain="vpc")
-        try:
-            secondary_pub_ip_0 = allocation_0["PublicIp"]
-            secondary_pub_ip_1 = allocation_1["PublicIp"]
+        # Reboot to update network config
+        client.execute("cloud-init clean --logs")
+        client.restart()
 
-            association_0 = ec2.associate_address(
-                AllocationId=allocation_0["AllocationId"],
-                NetworkInterfaceId=nic_id,
-                PrivateIpAddress=secondary_priv_ip_0,
-            )
-            assert association_0["ResponseMetadata"]["HTTPStatusCode"] == 200
-            association_1 = ec2.associate_address(
-                AllocationId=allocation_1["AllocationId"],
-                NetworkInterfaceId=nic_id,
-                PrivateIpAddress=secondary_priv_ip_1,
-            )
-            assert association_1["ResponseMetadata"]["HTTPStatusCode"] == 200
+        log_content = client.read_from_file("/var/log/cloud-init.log")
+        verify_clean_log(log_content)
+        verify_clean_boot(client)
 
-            # Reboot to update network config
-            client.execute("cloud-init clean --logs")
-            client.restart()
-
-            log_content = client.read_from_file("/var/log/cloud-init.log")
-            verify_clean_log(log_content)
-
-            # SSH over primary NIC works
-            instance_pub_ip = client.instance.ip
-            subp("nc -w 5 -zv " + instance_pub_ip + " 22", shell=True)
-
-            # SSH over secondary NIC works
-            subp("nc -w 5 -zv " + secondary_pub_ip_0 + " 22", shell=True)
-            subp("nc -w 5 -zv " + secondary_pub_ip_1 + " 22", shell=True)
-        finally:
-            with contextlib.suppress(Exception):
-                ec2.disassociate_address(
-                    AssociationId=association_0["AssociationId"]
-                )
-            with contextlib.suppress(Exception):
-                ec2.release_address(AllocationId=allocation_0["AllocationId"])
-            with contextlib.suppress(Exception):
-                ec2.disassociate_address(
-                    AssociationId=association_1["AssociationId"]
-                )
-            with contextlib.suppress(Exception):
-                ec2.release_address(AllocationId=allocation_1["AllocationId"])
+        # SSH over primary and secondary NIC works
+        for ip in public_ips:
+            subp("nc -w 5 -zv " + ip + " 22", shell=True)
 
 
 @pytest.mark.adhoc  # costly instance not available in all regions / azs
@@ -497,6 +459,7 @@ def test_ec2_multi_network_cards(setup_image, session_cloud: IntegrationCloud):
 
             log_content = client.read_from_file("/var/log/cloud-init.log")
             verify_clean_log(log_content)
+            verify_clean_boot(client)
 
             # SSH over secondary NICs works
             subp("nc -w 5 -zv " + allocation_1["PublicIp"] + " 22", shell=True)
