@@ -17,9 +17,9 @@ from cloudinit.config.schema import (
     get_schema,
     validate_cloudconfig_schema,
 )
+from tests.helpers import cloud_init_project_dir
 from tests.unittests.helpers import (
     FilesystemMockingTestCase,
-    cloud_init_project_dir,
     skipUnlessJsonSchema,
 )
 from tests.unittests.util import MockDistro
@@ -261,6 +261,3 @@ class TestResolvConfSchema:
         else:
             with pytest.raises(SchemaValidationError, match=error_msg):
                 validate_cloudconfig_schema(config, get_schema(), strict=True)
-
-
-# vi: ts=4 expandtab
