@@ -1,4 +1,5 @@
 # This file is part of cloud-init. See LICENSE file for license information.
+import importlib.metadata
 import json
 import logging
 import re
@@ -7,7 +8,7 @@ from collections import namedtuple
 
 import pytest
 
-from cloudinit import subp
+from cloudinit import lifecycle, subp
 from cloudinit.config.cc_ubuntu_pro import (
     _attach,
     _auto_attach,
@@ -23,7 +24,6 @@ from cloudinit.config.schema import (
     get_schema,
     validate_cloudconfig_schema,
 )
-from cloudinit.util import Version
 from tests.unittests.helpers import does_not_raise, mock, skipUnlessJsonSchema
 from tests.unittests.util import get_cloud
 
@@ -445,16 +445,17 @@ class TestUbuntuProSchema:
                     SchemaValidationError,
                     match=re.escape(
                         "ubuntu_advantage:  Deprecated in version 24.1."
-                        " Use ``ubuntu_pro`` instead"
+                        " Use **ubuntu_pro** instead"
                     ),
                 ),
-                # If __version__ no longer exists on jsonschema, that means
-                # we're using a high enough version of jsonschema to not need
-                # to skip this test.
-                JSONSCHEMA_SKIP_REASON
-                if Version.from_str(getattr(jsonschema, "__version__", "999"))
-                < Version(4)
-                else "",
+                (
+                    JSONSCHEMA_SKIP_REASON
+                    if lifecycle.Version.from_str(
+                        importlib.metadata.version("jsonschema")
+                    )
+                    < lifecycle.Version(4)
+                    else ""
+                ),
                 id="deprecation_of_ubuntu_advantage_skip_old_json",
             ),
             # Strict keys
@@ -1120,9 +1121,9 @@ class TestShouldAutoAttach:
         m_should_auto_attach.should_auto_attach.side_effect = (
             FakeUserFacingError("Some error")  # noqa: E501
         )
-        sys.modules[
-            "uaclient.api.u.pro.attach.auto.should_auto_attach.v1"
-        ] = m_should_auto_attach
+        sys.modules["uaclient.api.u.pro.attach.auto.should_auto_attach.v1"] = (
+            m_should_auto_attach
+        )
         assert not _should_auto_attach({})
         assert "Error during `should_auto_attach`: Some error" in caplog.text
         assert (
@@ -1144,20 +1145,18 @@ class TestShouldAutoAttach:
         self, ua_section, expected_result, caplog, fake_uaclient
     ):
         m_should_auto_attach = mock.Mock()
-        sys.modules[
-            "uaclient.api.u.pro.attach.auto.should_auto_attach.v1"
-        ] = m_should_auto_attach
+        sys.modules["uaclient.api.u.pro.attach.auto.should_auto_attach.v1"] = (
+            m_should_auto_attach
+        )
         should_auto_attach_value = object()
         m_should_auto_attach.should_auto_attach.return_value.should_auto_attach = (  # noqa: E501
             should_auto_attach_value
         )
-        if expected_result is None:  # Pro API does respond
+        if expected_result is None:
+            # Pro API does respond
             assert should_auto_attach_value == _should_auto_attach(ua_section)
-            assert (
-                "Checking if the instance can be attached to Ubuntu Pro took"
-                in caplog.text
-            )
-        else:  # cloud-init does respond
+        else:
+            # cloud-init does respond
             assert expected_result == _should_auto_attach(ua_section)
             assert not caplog.text
 
@@ -1173,9 +1172,9 @@ class TestAutoAttach:
         m_full_auto_attach.full_auto_attach.side_effect = FakeUserFacingError(
             "Some error"
         )
-        sys.modules[
-            "uaclient.api.u.pro.attach.auto.full_auto_attach.v1"
-        ] = m_full_auto_attach
+        sys.modules["uaclient.api.u.pro.attach.auto.full_auto_attach.v1"] = (
+            m_full_auto_attach
+        )
         expected_msg = "Error during `full_auto_attach`: Some error"
         with pytest.raises(RuntimeError, match=re.escape(expected_msg)):
             _auto_attach(self.ua_section)
@@ -1184,11 +1183,10 @@ class TestAutoAttach:
     def test_happy_path(self, caplog, mocker, fake_uaclient):
         mocker.patch.dict("sys.modules")
         sys.modules["uaclient.config"] = mock.Mock()
-        sys.modules[
-            "uaclient.api.u.pro.attach.auto.full_auto_attach.v1"
-        ] = mock.Mock()
+        sys.modules["uaclient.api.u.pro.attach.auto.full_auto_attach.v1"] = (
+            mock.Mock()
+        )
         _auto_attach(self.ua_section)
-        assert "Attaching to Ubuntu Pro took" in caplog.text
 
 
 class TestAttach:
