@@ -47,6 +47,7 @@ class TestUpgrade:
             "seed",
             "seed_dir",
         },
+        "CloudCIX": {"_metadata_url", "_net_cfg"},
         "CloudSigma": {"cepko", "ssh_public_key"},
         "CloudStack": {
             "api_ver",
@@ -186,8 +187,12 @@ class TestUpgrade:
         "match_case_insensitive_module_name",
         lambda name: f"DataSource{name}",
     )
+    @mock.patch(
+        "cloudinit.sources.DataSourceCloudStack.get_vr_address",
+        return_value="data-server.",
+    )
     def test_all_ds_init_vs_unpickle_attributes(
-        self, mode, mocker, paths, tmpdir
+        self, m_get_vr_address, mode, mocker, paths, tmpdir
     ):
         """Unpickle resets any instance attributes created in __init__
 
@@ -279,14 +284,15 @@ class TestUpgrade:
         paths = previous_obj_pkl.paths
         ds = ds_class(sys_cfg, distro, paths)
         if ds.dsname == "NoCloud" and previous_obj_pkl.__dict__:
-            expected = (
-                set({"seed_dirs"}),  # LP: #1568150 handled with getattr checks
-                set(),
-            )
+            # seed_dirs is covered by _unpickle
+            # _network_config and _network_eni were already initialized
+            # outside of __init__ so shouldn't need unpickling
+            expected = {"seed_dirs", "_network_config", "_network_eni"}
         else:
             expected = (set(),)
         missing_attrs = ds.__dict__.keys() - previous_obj_pkl.__dict__.keys()
-        assert missing_attrs in expected
+        for attr in missing_attrs:
+            assert attr in expected
 
     def test_networking_set_on_distro(self, previous_obj_pkl):
         """We always expect to have ``.networking`` on ``Distro`` objects."""
