@@ -1,10 +1,9 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 """Try to read a YAML file and report any errors.
 """
 
 import sys
-
 import yaml
 
 
@@ -13,11 +12,11 @@ if __name__ == "__main__":
     for fn in sys.argv[1:]:
         sys.stdout.write("%s" % (fn))
         try:
-            fh = open(fn, 'r')
-            yaml.safe_load(fh.read())
+            fh = open(fn, "rb")
+            yaml.safe_load(fh.read().decode("utf-8"))
             fh.close()
             sys.stdout.write(" - ok\n")
-        except Exception, e:
+        except Exception as e:
             sys.stdout.write(" - bad (%s)\n" % (e))
             bads += 1
     if bads > 0:
